@@ -28,8 +28,15 @@
 #    omniORB module -- omniORB specific things
 
 
-# $Id: __init__.py,v 1.26.2.22 2004/03/24 22:13:05 dgrisby Exp $
+# $Id: __init__.py,v 1.26.2.24 2005/03/02 11:32:44 dgrisby Exp $
 # $Log: __init__.py,v $
+# Revision 1.26.2.24  2005/03/02 11:32:44  dgrisby
+# New optional endian parameter to cdrMarshal / cdrUnmarshal.
+#
+# Revision 1.26.2.23  2005/01/24 17:28:57  dgrisby
+# Unbelievably unlikely race condition in thread cache / Python worker
+# thread creation. Of course it happened anyway.
+#
 # Revision 1.26.2.22  2004/03/24 22:13:05  dgrisby
 # Support reinitialising when the Python interpreter isn't finalized.
 #
@@ -344,33 +351,45 @@ sys.modules."""
     return ret
 
 
-def cdrMarshal(tc, data):
-    """cdrMarshal(TypeCode, data) -> binary string
+def cdrMarshal(tc, data, endian=-1):
+    """cdrMarshal(TypeCode, data [,endian]) -> binary string
 
 Marshal data with the given type into a CDR encapsulation. The data
 can later be converted back into Python objects with cdrUnmarshal().
 The encapsulation is language, platform, and ORB independent.
 
+If the endian boolean is provided, it represents the endianness to
+marshal with: True for little endian; false for big endian. The
+resulting string in this case is the raw marshalled form, not a CDR
+encapsulation. To unmarshal it, the endianness must be known.
+
 Throws CORBA.BAD_PARAM if the data does not match the TypeCode."""
 
     if not isinstance(tc, CORBA.TypeCode):
         raise TypeError("Argument 1 must be a TypeCode")
 
-    return _omnipy.cdrMarshal(tc._d, data)
+    return _omnipy.cdrMarshal(tc._d, data, endian)
 
-def cdrUnmarshal(tc, encap):
-    """cdrUnmarshal(TypeCode, string) -> data
+def cdrUnmarshal(tc, encap, endian=-1):
+    """cdrUnmarshal(TypeCode, string [,endian]) -> data
 
 Unmarshal a CDR stream created with cdrMarshal() or equivalent. The
 encapsulation must adhere to the given TypeCode.
 
+If the endian boolean is provided, it represents the endianness to
+unmarshal with: True for little endian; false for big endian. In this
+case, the string should be the raw marshalled form, not a CDR
+encapsulation. If the endianness does not match that used for
+marshalling, invalid data may be returned, or exceptions raised.
+
 Throws CORBA.MARSHAL if the binary string does not match the
 TypeCode."""
 
     if not isinstance(tc, CORBA.TypeCode):
         raise TypeError("Argument 1 must be a TypeCode")
 
-    return _omnipy.cdrUnmarshal(tc._d, encap)
+    return _omnipy.cdrUnmarshal(tc._d, encap, endian)
+
 
 WTHREAD_CREATED = 0
 WTHREAD_DELETED = 1
@@ -783,15 +802,16 @@ class WorkerThread(threading.Thread):
     hooks = []
 
     def __init__(self):
-        _thr_init(self, name="omniORB")
+        id = _thr_id()
+        _thr_init(self, name="omniORB-%d" % id)
         self._Thread__started = 1
-        self.id = _thr_id()
+        self.id = id
         _thr_acq()
-        if _thr_act.has_key(self.id):
+        if _thr_act.has_key(id):
             self.add = 0
         else:
             self.add = 1
-            _thr_act[self.id] = self
+            _thr_act[id] = self
         _thr_rel()
         if self.add:
             for hook in self.hooks:
@@ -802,8 +822,10 @@ class WorkerThread(threading.Thread):
             for hook in self.hooks:
                 hook(WTHREAD_DELETED, self)
             _thr_acq()
-            del _thr_act[self.id]
-            _thr_rel()
+            try:
+                del _thr_act[self.id]
+            finally:
+                _thr_rel()
 
     def _set_daemon(self): return 1
     def join(self):        assert 0, "cannot join an omniORB WorkerThread"
